/**
 * Copyright 2012-2017 Red Hat, Inc.
 *
 * Thermostat is distributed under the GNU General Public License,
 * version 2 or any later version (with a special exception described
 * below, commonly known as the "Classpath Exception").
 *
 * A copy of GNU General Public License (GPL) is included in this
 * distribution, in the file COPYING.
 *
 * Linking Thermostat code with other modules is making a combined work
 * based on Thermostat.  Thus, the terms and conditions of the GPL
 * cover the whole combination.
 *
 * As a special exception, the copyright holders of Thermostat give you
 * permission to link this code with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on Thermostat code.  If you modify Thermostat, you may
 * extend this exception to your version of the software, but you are
 * not obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 */

import * as angular from "angular";
import * as url from "url";

import { IAuthService } from "./auth-service.interface";

export class KeycloakAuthService implements IAuthService {

  private scope: ng.IRootScopeService;

  public constructor(private keycloak: any) {
  }

  public set rootScope(rootScope: ng.IRootScopeService) {
    this.scope = rootScope;
  }

  public login(): void {
    // no-op
  }

  public goToLogin(promise: ng.IDeferred<{}>): void {
    this.keycloak.login();
    this.scope.$broadcast("userLoginChanged");
    promise.resolve();
  }

  public logout(callback = angular.noop): void {
    this.scope.$broadcast("userLoginChanged");
    this.keycloak.logout();
    callback();
  }

  public status(): boolean {
    return this.keycloak.authenticated;
  }

  public refresh(): ng.IPromise<{}> {
    // refresh the token if it expires within 300 seconds
    return this.keycloak.updateToken(300);
  }

  public get authHeader(): string {
    return "Bearer " + this.keycloak.token;
  }

  public get username(): string {
    return this.keycloak.idTokenParsed.preferred_username;
  }

  public getCommandChannelUrl(baseUrl: string): string {
    const parsed = url.parse(baseUrl);
    parsed.query = { access_token: this.keycloak.token };
    return url.format(parsed);
  }

}
